@@ -109,8 +109,9 @@ int leaper_move_flags_to_betza(move_flag_t flags, char *buffer, size_t size) con
    }
 
    /* Simple leapers */
-   if (is_simple_leaper(flags)) {
-      int index = get_leaper_index(flags);
+   if (is_simple_leaper(flags) || is_double_leaper(flags)) {
+      int index  = get_leaper_index(flags);
+      int index2 = get_leaper_index2(flags);
 
       /* Some combination of simple leapers, this is a bit complicated.
        * We use the following algorithm:
@@ -194,6 +195,8 @@ int leaper_move_flags_to_betza(move_flag_t flags, char *buffer, size_t size) con
        */
       bitboard_t<kind> kb = movegen.make_leaper_bitboard(square, 1, 1)|movegen.make_leaper_bitboard(square, 1, 0);
       if (leaper == kb) {
+         if (index2 == index)
+            n += snprintf(buffer+n, size-n, "a");
          n += snprintf(buffer+n, size-n, "K");
          leaper.clear();
       }
@@ -274,7 +277,12 @@ int leaper_move_flags_to_betza(move_flag_t flags, char *buffer, size_t size) con
                   leaper &= ~(best_leaper & sym8[best]);
                }
             }
-            n += snprintf(buffer+n, size-n, "%s%s", block, leaper_description[best].txt);
+            if (is_double_leaper(flags)) {
+               if (index2 == index)
+                  n += snprintf(buffer+n, size-n, "%sa%s%s", block, block, leaper_description[best].txt);
+            } else {
+               n += snprintf(buffer+n, size-n, "%s%s", block, leaper_description[best].txt);
+            }
          }
 
          leaper &= ~best_leaper;
@@ -466,6 +474,7 @@ const char *piece_moves_to_betza(int piece, char *buffer = NULL, size_t size = 2
    char m_string[256] = { 0 };
    char c_string[256] = { 0 };
    char i_string[256] = { 0 };
+   char b_string[256] = { 0 };
    static char static_buffer[768] = { 0 };
    char *s;
    int n = 0;
@@ -477,11 +486,13 @@ const char *piece_moves_to_betza(int piece, char *buffer = NULL, size_t size = 2
 
    move_flag_t move    = pt.piece_move_flags[piece];
    move_flag_t capture = pt.piece_capture_flags[piece];
-   move_flag_t initial = pt.piece_special_move_flags[piece];
+   move_flag_t special = pt.piece_special_move_flags[piece];
+   move_flag_t initial = pt.piece_initial_move_flags[piece];
 
    move_flags_to_betza(move,    m_string, sizeof(m_string));
    move_flags_to_betza(capture, c_string, sizeof(c_string));
-   move_flags_to_betza(initial, i_string, sizeof(i_string));
+   move_flags_to_betza(special, i_string, sizeof(i_string));
+   move_flags_to_betza(initial, b_string, sizeof(b_string));
 
    /* Eliminate duplicates between the normal move and the initial move */
    if ((s = strstr(i_string, m_string))) {
@@ -495,6 +506,17 @@ const char *piece_moves_to_betza(int piece, char *buffer = NULL, size_t size = 2
          if (s[k] == 0) break;
       }
    }
+   if ((s = strstr(b_string, m_string))) {
+      size_t n1 = strlen(m_string);
+      size_t n2 = strlen(b_string);
+      char *p = s + n1;
+      if (!isdigit(p[0]))
+      for (size_t k = 0; k <= (n2-n1); k++) {
+         s[k] = p[k];
+         s[k+1] = 0;
+         if (s[k] == 0) break;
+      }
+   }
 
    if (move == capture) {
       strncpy(s_string, m_string, sizeof s_string);
@@ -536,6 +558,12 @@ const char *piece_moves_to_betza(int piece, char *buffer = NULL, size_t size = 2
       }
    }
 
+   if (strstr(s_string, "a")) {
+      char *s = strstr(s_string, "a")+1;
+      s[-1] = 0;
+      n += snprintf(buffer+n, size-n, "mcpa%s", s);
+      n += snprintf(buffer+n, size-n, "mcab%s", s);
+   }
 
    /* Common between move and capture */
    n += snprintf(buffer+n, size-n, "%s", s_string);
@@ -567,9 +595,11 @@ const char *piece_moves_to_betza(int piece, char *buffer = NULL, size_t size = 2
       buffer[n] = 0;
    }
 
-   /* Initial */
-   if (initial) {
+   /* Initial and special moves */
+   if (special) {
       n += snprintf(buffer+n, size-n, "i");
+      if (initial)
+         n += snprintf(buffer+n, size-n, "i");
 
       size_t l = strlen(i_string);
       for (size_t k=0; k<l; k++) {
@@ -583,22 +613,72 @@ const char *piece_moves_to_betza(int piece, char *buffer = NULL, size_t size = 2
          buffer[n] = 0;
       }
    }
+   if (initial) {
+      n += snprintf(buffer+n, size-n, "i");
+
+      size_t l = strlen(b_string);
+      for (size_t k=0; k<l; k++) {
+         if (isupper(b_string[k]) || strchr(post_movetype, b_string[k])) {
+            buffer[n++] = 'm';
+            buffer[n] = 0;
+         }
+         buffer[n++] = b_string[k];
+         if (strchr(post_movetype, b_string[k]))
+            buffer[n++] = b_string[++k];
+         buffer[n] = 0;
+      }
+   }
+
 
    if (pt.piece_flags[piece] & PF_CASTLE) {
-      bitboard_t<kind> side = bitboard_t<kind>::board_east_edge | bitboard_t<kind>::board_west_edge;
-      bitboard_t<kind> bb = movegen.castle_mask[SHORT][WHITE] & movegen.castle_mask[LONG][WHITE];
-      int s1 = bb.bitscan();
-      int s2 = movegen.castle_king_dest[SHORT][WHITE];
+      uint32_t castle_range[NUM_CASTLE_MOVES+1] = { 0 };
 
-      n += snprintf(buffer+n, size-n, "%ss", (initial) ? "" : "i");
+      for (int c = SHORT; c<NUM_CASTLE_MOVES; c++) {
+         bitboard_t<kind> kd = movegen.castle_king_dest[c][WHITE];
+         while (!kd.is_empty()) {
+            bitboard_t<kind> bb = movegen.castle_mask[SHORT][WHITE] & movegen.castle_mask[LONG][WHITE];
+            int s1 = bb.bitscan();
+            int s2 = kd.bitscan();
+            int r  = abs(s2 - s1);
 
-      bb = movegen.castle_mask[SHORT][WHITE];
-      bb.reset(s1);
-      int s3 = bb.bitscan();
-      if ((side & bb).is_empty() && bitboard_t<kind>::board_all.test(s3+1))
-         n += snprintf(buffer+n, size-n, "j");
+            castle_range[c] |= 1<<r;
+            kd.reset(s2);
+         }
+      }
 
-      n += snprintf(buffer+n, size-n, "O%d", abs(s2-s1));
+      castle_range[NUM_CASTLE_MOVES] = castle_range[SHORT] & castle_range[LONG];
+      castle_range[SHORT] &= ~castle_range[NUM_CASTLE_MOVES];
+      castle_range[LONG]  &= ~castle_range[NUM_CASTLE_MOVES];
+
+      for (int c = SHORT; c<NUM_CASTLE_MOVES; c++) {
+         bitboard_t<kind> side = bitboard_t<kind>::board_east_edge | bitboard_t<kind>::board_west_edge;
+         bitboard_t<kind> kd = movegen.castle_king_dest[c][WHITE];
+         while (!kd.is_empty()) {
+            bitboard_t<kind> bb = movegen.castle_mask[SHORT][WHITE] & movegen.castle_mask[LONG][WHITE];
+            int s1 = bb.bitscan();
+            int s2 = kd.bitscan();
+            int r  = abs(s2 - s1);
+
+            kd.reset(s2);
+
+            char ch = 's';
+            if (castle_range[c] & (1 << r)) ch = "rl"[c];
+            if (ch == 's' && c > SHORT) continue;
+
+            n += snprintf(buffer+n, size-n, "i%c", ch);
+
+            /* We need the "j" modifier if the castle partner is not a
+             * "corner piece".
+             */
+            bb = movegen.castle_mask[c][WHITE];
+            bb.reset(s1);
+            int s3 = bb.bitscan();
+            if ((side & bb).is_empty() && bitboard_t<kind>::board_all.test(s3+1))
+               n += snprintf(buffer+n, size-n, "j");
+
+            n += snprintf(buffer+n, size-n, "O%d", r);
+         }
+      }
    }
 
    if (strlen(buffer) == 0) {
