@@ -4,7 +4,7 @@ int move_mvvlva(move_t move) const
 
    if (is_capture_move(move)) {
       value += pt.see_piece_value[board.get_piece(get_move_capture_square(move))] * 8;
-      value -= pt.see_piece_value[get_move_piece(move)] / 8;
+      value -= pt.see_piece_value[board.get_piece(get_move_from(move))] / 8;
    }
 
    return value;
@@ -17,10 +17,10 @@ int move_value(move_t move, side_t /* side_to_move */) const
    if (is_capture_move(move))
       value += pt.see_piece_value[board.get_piece(get_move_capture_square(move))];
    if (is_drop_move(move))
-      value += pt.see_piece_value[get_move_piece(move)];
+      value += pt.see_piece_value[board.get_piece(get_move_from(move))];
    if (is_promotion_move(move)) {
       //value += pt.see_piece_value[get_move_promotion_piece(move)];
-      //value -= pt.see_piece_value[get_move_piece(move)];
+      //value -= pt.see_piece_value[board.get_piece(get_move_from(move))];
    }
    /* TODO: gate moves have an extra drop */
    return value;
@@ -85,9 +85,9 @@ int see(move_t move)
    int score[128];
    int depth;
    side_t side    = board.side_to_move;
-   int piece      = get_move_piece(move);
    int square     = get_move_to(move);
    int from       = get_move_from(move);
+   int piece      = board.get_piece(from);
    int last_piece = piece;
    bitboard_t<kind> attackers, hidden_attackers, pinned;
    bitboard_t<kind> xray_update;
